@@ -5,7 +5,7 @@ use warnings;
 
 use parent qw(IO::Async::Notifier);
 
-our $VERSION = '0.019';
+our $VERSION = '0.020';
 
 =head1 NAME
 
@@ -13,7 +13,7 @@ Net::Async::AMQP - provides client interface to AMQP using L<IO::Async>
 
 =head1 VERSION
 
-version 0.019
+version 0.020
 
 =head1 SYNOPSIS
 
@@ -658,21 +658,14 @@ sub close {
 		my @handler = (
 			close => sub {
 				my ($ev, $reason) = @_;
-				$f->done($reason) unless $f->is_ready;
 				$ev->unsubscribe;
+				return unless $f;
+				$f->done($reason) unless $f->is_ready;
 				weaken $f;
 			}
 		)
 	);
 
-	# ... and make sure we clean up after ourselves
-	$f->on_ready(sub {
-		$self->bus->unsubscribe_from_event(
-			@handler
-		);
-		weaken $f;
-	});
-
 	my $frame = Net::AMQP::Frame::Method->new(
 		method_frame => Net::AMQP::Protocol::Connection::Close->new(
 			reply_code => $args{code} // 320,
@@ -683,7 +676,14 @@ sub close {
 		'Connection::CloseOk' => [ $f, $self ],
 	);
 	$self->send_frame($frame);
-	return $f;
+
+	# ... and make sure we clean up after ourselves
+	$f->on_ready(sub {
+		$self->bus->unsubscribe_from_event(
+			@handler
+		);
+		weaken $f if $f;
+	});
 }
 
 =head2 channel_closed
